<?php

namespace filr;

/**
 * File System Class
 */
class FILR_Filesystem {
	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of FILR_Filesystem.
	 *
	 * @return object
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Constructor for FILR_Meta.
	 */
	public function __construct() {
		add_action( 'wp_ajax_upload_file', array( $this, 'ajax_upload_file' ) );
		add_action( 'wp_ajax_delete_file', array( $this, 'ajax_delete_file' ) );

		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			add_action( 'save_post_filr', array( $this, 'zip_files__premium_only' ), 20, 2 );
			add_action( 'check_file_acess', array( $this, 'check_file_access_on_interval' ) );
			add_action( 'init', array( $this, 'open_file' ) );
		}
	}

	public function zip_files__premium_only( $post_id, $post ) {
		$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

		// build the path.
		$uploads_directory = wp_upload_dir();
		$file_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $post_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $post_id );

		// Get all files from directory and count them.

		if ( ! is_dir( $file_directory ) ) {
			mkdir( $file_directory );
		}

		$files           = new \FilesystemIterator( $file_directory );
		$number_of_files = iterator_count( $files );

		if ( ! empty( $post_id ) ) {
			if ( $number_of_files > 1 ) {
				$file_url = Filr_ZIP::create_zip_file( $post_id, false );
				update_post_meta( $post_id, 'file-download', $file_url );
			}
		}
	}

	/**
	 * Ajax handler to upload files via FILR_Uploader.
	 *
	 * @return void
	 */
	public function ajax_upload_file() {

		$file_id    = sanitize_text_field( $_POST['post_id'] );
		$nonce      = $_POST['nonce'];
		$encryption = false;

		if ( ! wp_verify_nonce( $nonce, 'filr-uploader-nonce' ) ) {
			die();
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_post', $file_id ) ) {
			return $file_id;
		}

		$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

		// build the path.
		$uploads_directory = wp_upload_dir();
		$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );

		// if directory not exists, create one.
		if ( ! file_exists( $filr_directory ) ) {
			wp_mkdir_p( $filr_directory );
		}

		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			// wether or not file names are encrypted.
			$encryption = get_post_meta( $file_id, 'encrypt-filename', true );

			if ( $encryption ) {
				$uploader_encryption = 'auto';
			} else {
				$uploader_encryption = 'name';
			}
		} else {
			$uploader_encryption = 'name';
		}

		// upload file.
		$file     = 'file-upload';
		$uploader = new FILR_Uploader( $file, array(
			'uploadDir' => $filr_directory,
			'title'     => $uploader_encryption
		) );
		$data     = $uploader->upload();

		// check for existing files and combine them.
		$existing_files = get_post_meta( $file_id, 'file-upload', true );

		if ( isset( $existing_files['files'] ) && ! empty( $existing_files['files'] ) ) {
			foreach ( $existing_files['files'] as $file ) {
				$data['files'][] = $file;
			}
		}

		$file_url = '';

		if ( isset( $data['files'][0]['name'] ) ) {
			$file_url = $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . $file_id . '/' . $data['files'][0]['name'];
			update_post_meta( $file_id, 'file-download', $file_url );
		}


		// save file array for later usage.
		update_post_meta( $file_id, 'file-upload', $data );

		// clean file records.
		if ( apply_filters( 'filr_auto_clean', false ) ) {
			self::file_cleaner( $file_id );
		}

		$response = array( 'file-upload' => $data );

		if ( ! empty( $file_url ) ) {
			$response = array( 'file-upload' => $data, 'download_link' => $file_url );
		}

		print wp_json_encode( $response );
		exit;
	}

	/**
	 * Ajax handler to delete files via FILR_Uploader.
	 *
	 * @return void
	 */
	public function ajax_delete_file() {

		$file_id   = sanitize_text_field( $_POST['post_id'] );
		$file_name = sanitize_text_field( $_POST['filename'] );
		$nonce     = $_POST['nonce'];

		if ( ! wp_verify_nonce( $nonce, 'filr-uploader-nonce' ) ) {
			die();
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_post', $file_id ) ) {
			return $file_id;
		}

		$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

		// build the path.
		$uploads_directory = wp_upload_dir();
		$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );

		$file = $filr_directory . $file_name;

		// delete file from file upload array in post meta.
		$existing_files = get_post_meta( $file_id, 'file-upload', true );
		$new_files      = array();

		if ( $existing_files['files'] ) {
			foreach ( $existing_files['files'] as $file ) {
				if ( $file['name'] !== $file_name ) {
					$new_files[] = $file;
				}
			}
		}

		// Update new array of files in metadata.
		$existing_files['files'] = $new_files;
		update_post_meta( $file_id, 'file-upload', $existing_files );

		// if more than one file we need to regenerate zip too, otherwise update download link meta only.
		if ( $existing_files['files'] ) {
			$file_url = $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . $file_id . '/' . $existing_files['files'][0]['name'];
			update_post_meta( $file_id, 'file-download', $file_url );
		} else {
			delete_post_meta( $file_id, 'file-download' );
			if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
				// delete zip meta if exists.
				delete_post_meta( $file_id, 'file-zip' );
			}
		}

		// delete the file from the filesystem.
		if ( isset( $file_name ) ) {
			$deletable_file = $filr_directory . str_replace( array( DIRECTORY_SEPARATOR, '\\' ), '', $file_name );

			if ( file_exists( $deletable_file ) ) {
				unlink( $deletable_file );
			}
		}

		// clean file records.
		if ( apply_filters( 'filr_auto_clean', false ) ) {
			self::file_cleaner( $file_id );
		}

		$response = array( 'delete' => true );

		if ( ! empty( $file_url ) ) {
			$response = array( 'delete' => true, 'download_link' => $file_url );
		}
		print wp_json_encode( $response );
		exit;
	}

	/**
	 * Handles the creation of a subfolder inside of /uploads/ directory.
	 *
	 * @return void
	 */
	public static function create_filr_directory( $directory_name ) {
		$uploads_directory = wp_upload_dir();
		$filr_directory    = apply_filters( 'filr_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $directory_name, $directory_name );

		if ( ! file_exists( $filr_directory ) ) {
			wp_mkdir_p( $filr_directory );
		}
	}

	/**
	 * Delete a subfolder inside of /uploads/ directory.
	 *
	 * @return void
	 */
	public static function delete_filr_directory() {
		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$filr_directory    = apply_filters( 'filr_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'], $settings['filr_download_directory'] );

		if ( file_exists( $filr_directory ) ) {
			$wp_filesystem->delete( $filr_directory, true );
		}
	}

	/**
	 * Creates an index.php file to prevent direct access via HTTP.
	 *
	 * @return void
	 */
	public static function create_index_file() {

		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$file              = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . 'index.php';
		$content           = '<?php // silence is golden.';

		if ( ! file_exists( $file ) ) {
			$wp_filesystem->put_contents( $file, $content, FS_CHMOD_FILE );
		}
	}

	/**
	 * Delete a the index file
	 *
	 * @return void
	 */
	public static function delete_index_file() {

		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$file              = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . 'index.php';

		if ( file_exists( $file ) ) {
			$wp_filesystem->delete( $file, true );
		}
	}

	/**
	 * Creates an .htaccess file to prevent direct access via HTTP.
	 *
	 * @return void
	 */
	public static function create_htaccess_file() {

		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$file              = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . '/.htaccess';
		$content           = 'Options All -Indexes';

		if ( ! file_exists( $file ) ) {
			$wp_filesystem->put_contents( $file, $content, FS_CHMOD_FILE );
		}
	}

	/**
	 * Delete a the .htaccess file
	 *
	 * @return void
	 */
	public static function delete_htaccess_file() {
		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$file              = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . '/.htaccess';

		if ( file_exists( $file ) ) {
			$wp_filesystem->delete( $file, true );
		}
	}

	/**
	 * Returns unit based on byte size.
	 *
	 * @param int $bytes number of bytes.
	 *
	 * @return string
	 */
	public static function format_byte_sizes( int $bytes ): string {
		if ( $bytes >= 1073741824 ) {
			$bytes = number_format( $bytes / 1073741824, 2 ) . ' GB';
		} elseif ( $bytes >= 1048576 ) {
			$bytes = number_format( $bytes / 1048576, 2 ) . ' MB';
		} elseif ( $bytes >= 1024 ) {
			$bytes = number_format( $bytes / 1024, 2 ) . ' KB';
		} elseif ( $bytes > 1 ) {
			$bytes = $bytes . ' bytes';
		} elseif ( $bytes == 1 ) {
			$bytes = $bytes . ' byte';
		} else {
			$bytes = '0 bytes';
		}

		return $bytes;
	}

	/**
	 * Block file access via htaccess
	 *
	 * @param int $file_id current download object id.
	 * @param string $expire_date a date string for expiration.
	 * @param string $remaining_downloads number of remaining downloads available.
	 *
	 * @return bool
	 */
	public static function block_file_access( int $file_id, string $expire_date, string $remaining_downloads ): bool {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {

			if ( empty( $expire_date ) && empty( $remaining_downloads ) ) {
				return false;
			}

			// check if file systems works.
			global $wp_filesystem;

			if ( ! function_exists( 'WP_Filesystem' ) ) {
				require_once( ABSPATH . 'wp-admin/includes/file.php' );
			}

			if ( is_null( $wp_filesystem ) ) {
				WP_Filesystem();
			}

			// build the path.
			$uploads_directory = wp_upload_dir();
			$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
			$file_directory    = $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR;

			// get download url.
			$download_url = get_post_meta( $file_id, 'file-download', true );

			// build htaccess file.
			$htaccess_file = $file_directory . '.htaccess';
			$content       = 'RewriteRule ^(.*)$ ' . $download_url . ' [F,L]';

			// check for expire date first.
			$expiration_date = new \DateTime( $expire_date );
			$today           = new \DateTime( 'now' );

			do_action( 'filr_before_check_expire_date', $expiration_date, $today );

			if ( ! empty( $expire_date ) ) {
				if ( $today->getTimestamp() >= $expiration_date->getTimestamp() ) {
					if ( ! file_exists( $htaccess_file ) ) {
						$wp_filesystem->put_contents( $htaccess_file, $content, FS_CHMOD_FILE );
					}

					return true;
				} else {
					if ( file_exists( $htaccess_file ) ) {
						$wp_filesystem->delete( $htaccess_file, true );
					}
				}
			}

			// check for remaining downloads now.
			if ( ! empty( $remaining_downloads ) && $remaining_downloads < 1 ) {
				if ( ! file_exists( $htaccess_file ) ) {
					$wp_filesystem->put_contents( $htaccess_file, $content, FS_CHMOD_FILE );
				}

				return true;
			} else {
				if ( file_exists( $htaccess_file ) ) {
					$wp_filesystem->delete( $htaccess_file, true );
				}
			}
		}

		return false;
	}

	/**
	 * Check if files has to be blocked per interval (hourly).
	 *
	 * @return void
	 */
	public function check_file_access_on_interval() {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$args = array(
				'post_type'      => 'filr',
				'posts_per_page' => - 1,
				'fields'         => 'ids',
			);

			$files = get_posts( $args );

			foreach ( $files as $file_id ) {
				$expire_download = get_post_meta( $file_id, 'expire-download', true );
				$expire_date     = get_post_meta( $file_id, 'expire-date', true );

				// check if file is blocked.
				self::block_file_access( $file_id, $expire_date, $expire_download );
			}
		}
	}

	/**
	 * Clean file records with filesystem comparing.
	 *
	 * @param int $file_id current file id.
	 *
	 * @return void
	 */
	public static function file_cleaner( int $file_id ) {
		$file_upload       = get_post_meta( $file_id, 'file-upload', true );
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$uploads_directory = wp_upload_dir();
		$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );
		$remaining_files   = list_files( $filr_directory );
		$current_files     = array();

		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			// check if zip.
			$zip = get_post_meta( $file_id, 'file-zip', true );

			if ( $zip ) {
				$current_files[] = $zip;
			}
		}

		foreach ( $file_upload['files'] as $file ) {
			$current_files[] = $file['file'];
		}

		foreach ( $remaining_files as $file ) {
			if ( ! in_array( $file, $current_files ) ) {
				unlink( $file );
			}
		}
	}

	/**
	 * Clean file records with filesystem comparing.
	 *
	 * @param int $file_id current file id.
	 *
	 * @return void
	 */
	public static function check_encryption_mode( int $file_id, $encrypt ) {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {

			// file system data.
			$file_upload       = get_post_meta( $file_id, 'file-upload', true );
			$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
			$uploads_directory = wp_upload_dir();
			$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );
			$filr_url          = apply_filters( 'filr_file_url', $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . $file_id . '/', $settings['filr_download_directory'], $file_id );

			// rename files.
			$renamed_files = array();

			if ( ! empty( $file_upload['files'] ) ) {
				foreach ( $file_upload['files'] as $file ) {
					if ( ! file_exists( $file['file'] ) ) {
						continue;
					}
					if ( true === $encrypt ) {
						$new_filename = substr( str_shuffle( '_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' ), 0, 12 ) . '.' . $file['extension'];
						rename( $file['file'], $filr_directory . $new_filename );
						$file['name'] = $new_filename;
						$file['file'] = $filr_directory . $new_filename;
					} else {
						$new_filename = $file['old_name'];
						rename( $file['file'], $filr_directory . $new_filename );
						$file['name'] = $new_filename;
						$file['file'] = $filr_directory . $new_filename;
					}
					$renamed_files[] = $file;
				}

				$file_upload['files'] = $renamed_files;

				update_post_meta( $file_id, 'file-upload', $file_upload );

				// rename zips.
				$zip = get_post_meta( $file_id, 'file-zip', true );

				if ( $zip ) {
					if ( apply_filters( 'filr_auto_clean', false ) ) {
						self::file_cleaner( $file_id );
					}
					if ( true === $encrypt ) {
						$file_url = Filr_ZIP::create_zip_file( $file_id, true );
						update_post_meta( $file_id, 'file-zip', $file_url );
						update_post_meta( $file_id, 'file-download', $file_url );
					} else {
						$file_url = Filr_ZIP::create_zip_file( $file_id, false );
						update_post_meta( $file_id, 'file-zip', $file_url );
						update_post_meta( $file_id, 'file-download', $file_url );
					}
				} else {
					update_post_meta( $file_id, 'file-download', $filr_url . $file_upload['files'][0]['name'] );
				}
			}
		}
	}

	/**
	 * Open file and force download with PHP.
	 *
	 * @return void
	 */
	public function open_file() {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

			if ( isset( $_GET[ $settings['filr_download_directory'] ] ) && ! empty( $_GET[ $settings['filr_download_directory'] ] ) ) {
				$file_id = $_GET[ $settings['filr_download_directory'] ];

				if ( 'on' === $settings['filr_secure_download_links_encryption'] ) {
					$file_id = base64_decode( $_GET[ $settings['filr_download_directory'] ] );
				}

				$data = get_post_meta( $file_id, 'file-upload', true );

				if ( ! empty( $data['files'] ) ) {
					$file = $data['files'][0]['file'];
				} else {
					// It's a frontend uploaded file.
					$uploads_directory = wp_upload_dir();
					$file              = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id ) . $file_id . '.zip';
				}

				// Check if it's a ZIP file.
				$zip = get_post_meta( $file_id, 'file-zip', true );

				if ( $zip ) {
					// Convert URL to local path and set new file object.
					$file = str_replace( untrailingslashit( \get_bloginfo( 'url' ) ), ABSPATH, $zip );
				}

				if ( file_exists( $file ) ) {
					// Check file access.
					$expire_date     = get_post_meta( $file_id, 'expire-date', true );
					$expire_download = get_post_meta( $file_id, 'expire-download', true );

					// check if file blocked.
					$file_blocked = self::block_file_access( $file_id, $expire_date, $expire_download );

					if ( $file_blocked ) {
						echo esc_html( $settings['filr_access_error_message'] );
						exit;
					}

					// check if user allowed to get file.
					$allowed_emails = get_post_meta( $file_id, 'restrict-user', true );
					$allowed_role   = get_post_meta( $file_id, 'restrict-user-role', true );
					$user           = wp_get_current_user();
					$user_allowed   = true;

					// clean email field.
					if ( strpos( $allowed_emails, ' ' ) ) {
						str_replace( ' ', '', $allowed_emails );
					}

					// check allowed mail or mails.
					if ( isset( $allowed_emails ) && ! empty( $allowed_emails ) ) {
						if ( strpos( $allowed_emails, ',' ) ) {
							$allowed_emails = explode( ',', $allowed_emails );

							if ( ! in_array( $user->user_email, $allowed_emails, true ) ) {
								$user_allowed = false;
							}
						} else {
							if ( $user->user_email !== $allowed_emails ) {
								$user_allowed = false;
							}
						}
					}

					if ( isset( $allowed_role ) && ! empty( $allowed_role ) && 'no-role' !== $allowed_role ) {
						if ( ! in_array( $allowed_role, $user->roles, true ) ) {
							$user_allowed = false;
						}
					}

					if ( current_user_can( 'administrator' ) ) {
						$user_allowed = true;
					}

					if ( ! $user_allowed ) {
						echo esc_html( $settings['filr_access_error_message'] );
						exit;
					}

					// Hook to implement custom validation.
					$access_allowed = apply_filters( 'filr_access_allowed', true, $file_id, $user );

					if ( ! $access_allowed ) {
						// Custom redirect.
						$should_redirect = apply_filters( 'filr_redirect_access_denied', false, $file_id, $user );

						if ( $should_redirect ) {
							$redirect_url = apply_filters( 'filr_redirect_access_denied_url', home_url(), $file_id, $user );
							wp_redirect( $redirect_url );
						} else {
							echo esc_html( $settings['filr_access_error_message'] );
							exit;
						}
					}

					// Add hook for additional checkups.
					do_action( 'filr_allow_file_access', $file_id );

					// If we get this far provide the download and check if we need to reduce the amount of available downloads.
					$expire_download = get_post_meta( $file_id, 'expire-download', true );

					if ( ! empty( $expire_download ) ) {
						if ( $expire_download > 0 ) {
							update_post_meta( $file_id, 'expire-download', intval( $expire_download ) - 1 );
						} else {
							echo esc_html( $settings['filr_access_error_message'] );
							exit;
						}
					}

					header( 'Content-Description: File Transfer' );
					header( 'Content-Type: application/octet-stream' );
					header( 'Content-Disposition: attachment; filename="' . basename( $file ) . '"' );
					header( 'Expires: 0' );
					header( 'Cache-Control: must-revalidate' );
					header( 'Pragma: public' );
					header( 'Content-Length: ' . filesize( $file ) );
					readfile( $file );
					exit;
				}
			}
		}
	}

	/**
	 * Get secure download URL from id.
	 *
	 * @param int $post_id given post id.
	 *
	 * @return string|bool
	 */
	public static function get_secure_url( int $post_id ) {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			if ( empty( $post_id ) ) {
				return false;
			}

			$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

			if ( 'on' === $settings['filr_secure_download_links'] ) {
				$secure_url = untrailingslashit( get_bloginfo( 'url' ) ) . '?' . $settings['filr_download_directory'] . '=' . $post_id;

				if ( 'on' === $settings['filr_secure_download_links_encryption'] ) {
					$secure_url = untrailingslashit( get_bloginfo( 'url' ) ) . '?' . $settings['filr_download_directory'] . '=' . base64_encode( $post_id );
				}

				return $secure_url;
			}
		}

		return false;
	}
}
