<?php

namespace filr;

/**
 * Shortcode Class
 */
class FILR_Folder {

	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of FILR_Folder.
	 *
	 * @return object
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}


	/**
	 * Constructor for FILR_Folder.
	 */
	public function __construct() {
		add_shortcode( 'filr-folder', array( $this, 'add_shortcode' ) );
	}

	/**
	 * Register a shortcode for filr table display.
	 *
	 * @param array $atts array of possible attributes.
	 *
	 * @return string
	 */
	public function add_shortcode( array $atts ): string {

		$settings = wp_parse_args( get_option( 'filr_shortcode' ), FILR_Admin::get_defaults( 'filr_shortcode' ) );

		if ( ! isset( $atts['folder'] ) ) {
			return '';
		}

		$args = array(
			'numberposts' => - 1,
			'post_type'   => 'filr',
			'meta_key'    => 'assigned-folder',
			'meta_value'  => $atts['folder'],
			'post_status' => 'publish',
		);

		$files = get_posts( $args );

		$use_publish_date = false;

		// publish date title.
		if ( isset( $settings['filr_show_publish_instead_of_last_modified_date'] ) && ! empty( $settings['filr_show_publish_instead_of_last_modified_date'] ) && 'on' === $settings['filr_show_publish_instead_of_last_modified_date'] ) {
			add_filter(
				'filr_date_title',
				function ( $title ) {
					return esc_html__( 'Published', 'filr' );
				}
			);

			$use_publish_date = true;
		}

		// Build the dynamic header and rows.
		$header_data = apply_filters( 'filr_header_columns', array(
			'file'      => array(
				'title' => esc_html__( 'Files', 'filr' ),
				'hide'  => 'off',
			),
			'version'   => array(
				'title' => esc_html__( 'Version', 'filr' ),
				'hide'  => 'off',
			),
			'user'      => array(
				'title' => esc_html__( 'Uploaded by', 'filr' ),
				'hide'  => 'off',
			),
			'size'      => array(
				'title' => esc_html__( 'Size', 'filr' ),
				'hide'  => $settings['filr_hide_size_row'],
			),
			'type'      => array(
				'title' => esc_html__( 'Type', 'filr' ),
				'hide'  => $settings['filr_hide_type_row'],
			),
			'remaining' => array(
				'title' => esc_html__( 'Remaining', 'filr' ),
				'hide'  => $settings['filr_hide_remaining_row'],
			),
			'expire'    => array(
				'title' => esc_html__( 'Expires', 'filr' ),
				'hide'  => $settings['filr_hide_expires_row'],
			),
			'date'      => array(
				'title' => apply_filters( 'filr_date_title', esc_html__( 'Last Modified', 'filr' ) ),
				'hide'  => $settings['filr_hide_date_row'],
			),
			'download'  => array(
				'title' => esc_html__( 'Download', 'filr' ),
				'hide'  => 'off',
			),
		) );

		$headers = explode( '|', $settings['filr_shortcode_sort_rows'] );

		// folder title.
		$folder_name = '';

		if ( isset( $atts['folder-name'] ) && ! empty( $atts['folder-name'] ) ) {
			$folder_name = $atts['folder-name'];
		}

		ob_start();
		?>
        <table class="filr-folder-table" id="filr-folder-<?php echo esc_html( $atts['folder'] ); ?>">
            <caption><?php echo esc_html( $folder_name ); ?></caption>
            <thead>
            <tr>
				<?php foreach ( $headers as $th ) : ?>
					<?php if ( 'off' === $header_data[ $th ]['hide'] ) : ?>
                        <th class="<?php echo esc_attr( $th ); ?>"><?php echo esc_html( $header_data[ $th ]['title'] ); ?></th>
					<?php endif; ?>
				<?php endforeach; ?>
            </tr>
            </thead>
            <tbody>
			<?php foreach ( $files as $file ) : ?>
				<?php

				$file_id   = $file->ID;
				$link      = apply_filters( 'filr_download_link', get_post_meta( $file_id, 'file-download', true ) );
				$upload    = get_post_meta( $file_id, 'file-upload', true );
				$is_folder = false;

				// Check if secure links are used.
				$secure_url = FILR_Filesystem::get_secure_url( $file_id );

				if ( $secure_url ) {
					$link = $secure_url;
				}

				// Get parent file id.
				$parent_file_id = $file_id;

				// Get the title.
				$title = esc_html( $file->post_title );

				// Get file date.
				$file_date = esc_html( $file->post_date );

				// Download button.
				$button = esc_html( apply_filters( 'filr_replace_button_name', $file->post_title ) );

				if ( isset( $settings['filr_rename_download_button'] ) && ! empty( $settings['filr_rename_download_button'] ) ) {
					$button = apply_filters( 'filr_replace_button_name', $settings['filr_rename_download_button'] );
				}

				if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {

					// Check author of post.
					$user_name = apply_filters( 'filr_file_author', get_the_author_meta( 'display_name', $file->post_author ), $file->post_author );

					// Check for version numbers.
					$version = get_post_meta( $file_id, 'version', true );

					if ( empty( $version ) ) {
						$version = '';
					}

					$expire_download = get_post_meta( $file_id, 'expire-download', true );
					$expire_date     = get_post_meta( $file_id, 'expire-date', true );

					// check if file blocked.
					$file_blocked = FILR_Filesystem::block_file_access( $file_id, $expire_date, $expire_download );

					if ( $file_blocked ) {
						continue;
					}

					// check if user allowed to get file.
					$allowed_emails = get_post_meta( $file_id, 'restrict-user', true );
					$allowed_role   = get_post_meta( $file_id, 'restrict-user-role', true );
					$user           = wp_get_current_user();
					$user_allowed   = true;

					// clean email field.
					if ( strpos( $allowed_emails, ' ' ) ) {
						str_replace( ' ', '', $allowed_emails );
					}

					// check allowed mail or mails.
					if ( isset( $allowed_email ) && ! empty( $allowed_email ) ) {
						if ( strpos( $allowed_emails, ',' ) ) {
							$allowed_emails = explode( ',', $allowed_emails );

							if ( ! in_array( $user->user_email, $allowed_emails, true ) ) {
								$user_allowed = false;
							}
						} else {
							if ( $user->user_email !== $allowed_email ) {
								$user_allowed = false;
							}
						}
					}

					if ( isset( $allowed_role ) && ! empty( $allowed_role ) && 'no-role' !== $allowed_role ) {
						if ( ! in_array( $allowed_role, $user->roles, true ) ) {
							$user_allowed = false;
						}
					}

					if ( current_user_can( 'administrator' ) ) {
						$user_allowed = true;
					}

					if ( ! $user_allowed ) {
						continue;
					}

					// Hook to implement custom validation.
					$access_allowed = apply_filters( 'filr_access_allowed', true, $file_id, $user );

					if ( ! $access_allowed ) {
						continue;
					}

					if ( 'on' === $settings['filr_show_expired_downloads'] ) {
						add_filter( 'filr_show_blocked', '__return_false' );
					}

					// create date from expire_date.
					if ( empty( $expire_date ) ) {
						$expire_date = __( 'never', 'filr' );
					} else {
						$expire_date = date_create( $expire_date );
						$expire_date = date_format( $expire_date, $settings['filr_date_format'] );
					}

					// if no download limit is set, set to INF.
					if ( '' === $expire_download ) {
						$expire_download = '∞';
					}

					// Is a folder?
					$is_folder    = get_post_meta( $file_id, 'is-folder', true );
					$folder_class = '';

					if ( $is_folder ) {
						$folder_class = 'filr-folder';
					}

					// Is a external source?
					$is_external     = get_post_meta( $file_id, 'is-external', true );
					$external_source = '';

					if ( $is_external ) {
						$external_source = get_post_meta( $file_id, 'external-source', true );
					}

					// Check if secure links are used.
					$secure_url = FILR_Filesystem::get_secure_url( $file_id );

					if ( $secure_url && ! $is_folder ) {
						$link = $secure_url;
					}
				}

				// if more than one file it's a zip file.
				if ( isset( $upload['files'] ) && ! empty( $upload['files'] ) ) {
					if ( count( $upload['files'] ) > 1 ) {
						// if date parameter is set.
						$date      = apply_filters( 'filr_date_output', date_format( date_create( $upload['files'][0]['date'] ), $settings['filr_date_format'] ) );
						$size      = 0;
						$extension = 'ZIP';

						// calculate size of zip file.
						foreach ( $upload['files'] as $uploaded_file ) {
							$size = $size + $uploaded_file['size'];
						}
						$file = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';
					} else {
						$file = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button"  data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';

						// if date parameter is set.
						$date      = apply_filters( 'filr_date_output', date_format( date_create( $upload['files'][0]['date'] ), $settings['filr_date_format'] ) );
						$size      = $upload['files'][0]['size'];
						$extension = $upload['files'][0]['extension'];
					}

					// Publish date instead of last modified date.
					if ( isset( $use_publish_date ) && ! empty( $use_publish_date ) && 'on' === $use_publish_date ) {
						// Get publish date.
						$publish_date = date_format( date_create( $file_date ), $settings['filr_date_format'] );
						$date         = $publish_date;
					}
				}

				// if it's a folder.
				if ( $is_folder ) {
					$file      = '<a role="button" aria-label="' . esc_html__( 'Open Folder', 'filr' ) . '" class="filr-folder-button" id="folder-' . $parent_file_id . '">' . esc_html__( 'Open Folder', 'filr' ) . '</a>';
					$extension = esc_html__( 'Folder', 'filr' );
					$size      = '';
					$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
				}

				// if it's a external source.
				if ( $is_external ) {
					// Check for target blank.
					$target = '';

					if ( 'on' === $settings['filr_target_blank_external'] ) {
						$target = 'target="_blank" ';
					}

					$file      = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '"' . $target . ' href="' . $external_source . '" download>' . $button . '</a>';
					$extension = esc_html__( 'External File', 'filr' );
					$size      = get_post_meta( $parent_file_id, 'external-size', true );
					$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
				}

				// Frontend Upload.
				if ( $link && empty( $upload['files'] ) && ! $is_external ) {
					// Check for target blank.
					$target = '';

					if ( 'on' === $settings['filr_target_blank_external'] ) {
						$target = 'target="_blank" ';
					}

					$file      = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';
					$extension = __( 'ZIP', 'filr' );

					// Calculate file size.
					$uploads_directory = wp_upload_dir();
					$status            = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
					$link_file_path    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $status['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $status['filr_download_directory'], $file_id ) . $file_id . '.zip';


					if ( file_exists( $link_file_path ) ) {
						$link_size = FILR_Filesystem::format_byte_sizes( filesize( $link_file_path ) );
					} else {
						$link_size = 0;
					}

					$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
					$timestamp = date_format( date_create( $file_date ), 'Ymd' );
				}

				// If nothing matches skip file.
				if ( empty( $upload['files'] ) && ! $is_folder && ! $is_external && ! $link ) {
					continue;
				}

				?>
                <tr class="<?php echo esc_attr( $folder_class ); ?>">
					<?php foreach ( $headers as $td ) : ?>
						<?php if ( 'off' === $header_data[ $td ]['hide'] ) : ?>
							<?php if ( 'file' === $td ) : ?>
                                <td><?php echo esc_html( $title ); ?></td>
							<?php endif; ?>
							<?php if ( 'version' === $td ) : ?>
                                <td class="version"><?php echo esc_html( $version ); ?></td>
							<?php endif; ?>
							<?php if ( 'user' === $td ) : ?>
                                <td class="user"><?php echo esc_html( $user_name ); ?></td>
							<?php endif; ?>
							<?php if ( 'size' === $td ) : ?>
                                <td class="size">
									<?php if ( $is_folder )  : ?>
                                        <span>-</span>
									<?php elseif ( $is_external )  : ?>
										<?php echo esc_html( $size ); ?>
									<?php elseif ( $link && empty( $upload['files'] ) )  : ?>
										<?php echo esc_html( $link_size ); ?>
									<?php else : ?>
										<?php echo esc_html( FILR_Filesystem::format_byte_sizes( $size ) ); ?>
									<?php endif; ?>
                                </td>
							<?php endif; ?>
							<?php if ( 'type' === $td ) : ?>
                                <td><?php echo esc_html( strtoupper( $extension ) ); ?></td>
							<?php endif; ?>
							<?php if ( 'remaining' === $td ) : ?>
                                <td><?php echo esc_html( $expire_download ); ?></td>
							<?php endif; ?>
							<?php if ( 'expire' === $td ) : ?>
                                <td><?php echo esc_html( $expire_date ); ?></td>
							<?php endif; ?>
							<?php if ( 'date' === $td ) : ?>
                                <td><?php echo esc_html( $date ); ?></td>
							<?php endif; ?>
							<?php if ( 'download' === $td ) : ?>
                                <td><?php echo wp_kses_post( $file ); ?></td>
							<?php endif; ?>
							<?php do_action( 'filr_custom_column', $td, $file_id ); ?>
						<?php endif; ?>
					<?php endforeach; ?>
                </tr>
			<?php endforeach; ?>
            </tbody>
        </table>
		<?php
		return ob_get_clean();
	}
}
