<?php

namespace filr;

/**
 * FILR_Frontend_Uploader Class
 */
class FILR_Frontend_Uploader {

	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of FILR_Frontend_Uploader.
	 *
	 * @return object
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}


	/**
	 * Constructor for FILR_Frontend_Uploader.
	 */
	public function __construct() {
		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			add_shortcode( 'filr-uploader', array( $this, 'add_shortcode' ) );
			add_action( 'wp_footer', array( $this, 'add_dynamic_styles' ) );
			add_action( 'wp_enqueue_scripts', array( $this, 'add_public_scripts' ) );
			add_action( 'wp_ajax_frontend_upload_file', array( $this, 'frontend_upload_file' ) );
			add_action( 'wp_ajax_nopriv_frontend_upload_file', array( $this, 'frontend_upload_file' ) );
			add_action( 'wp_ajax_frontend_delete_file', array( $this, 'frontend_delete_file' ) );
			add_action( 'wp_ajax_nopriv_frontend_delete_file', array( $this, 'frontend_delete_file' ) );
			add_action( 'wp_ajax_frontend_create_post', array( $this, 'frontend_create_post' ) );
			add_action( 'wp_ajax_nopriv_frontend_create_post', array( $this, 'frontend_create_post' ) );
			add_action( 'filr_frontend_file_uploaded', array( $this, 'send_notification_email' ) );
		}
	}

	/**
	 * Add public scripts.
	 *
	 * @return void
	 */
	public function add_public_scripts() {
		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			if ( shortcode_exists( 'filr-uploader' ) ) {
				$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

				wp_enqueue_style( 'filr-frontend-uploader', FILR_URL . '/assets/filr-frontend-uploader' . $suffix . '.css', array(), FILR_VERSION, 'all' );
				wp_enqueue_script( 'filr-frontend-uploader', FILR_URL . '/assets/filr-frontend-uploader' . $suffix . '.js', array( 'jquery' ), FILR_VERSION, true );

				// Media Uploader.
				wp_enqueue_media();

				// uploader styles and scripts.
				wp_enqueue_style( 'font-fileuploader', FILR_URL . '/assets/font/font-fileuploader.min.css', array(), false, 'all' );
				wp_enqueue_style( 'fileuploader', FILR_URL . '/assets/jquery.fileuploader.min.css', array(), false, 'all' );
				wp_enqueue_script( 'fileuploader', FILR_URL . '/assets/jquery.fileuploader.min.js', array( 'jquery' ), false, true );

				$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
				$uploader_settings = wp_parse_args( get_option( 'filr_frontend_uploader' ), FILR_Admin::get_defaults( 'filr_frontend_uploader' ) );
				$uploads_directory = wp_upload_dir();
				$filr_dir          = $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . get_the_id() . '/';
				$translations      = array();

				if ( ! empty( $uploader_settings['filr_frontend_uploader_headline'] ) && ! empty( $uploader_settings['filr_frontend_uploader_limiter_text'] ) && ! empty( $uploader_settings['filr_frontend_uploader_button_label'] ) ) {
					$translations = array(
						$uploader_settings['filr_frontend_uploader_headline'],
						$uploader_settings['filr_frontend_uploader_limiter_text'],
						$uploader_settings['filr_frontend_uploader_button_label']
					);
				}

				wp_localize_script( 'filr-frontend-uploader', 'frontend_uploader', array(
					'ajax_url'                => admin_url( 'admin-ajax.php' ),
					'frontend_uploader_nonce' => wp_create_nonce( 'filr-frontend-uploader-nonce' ),
					'filr_dir'                => $filr_dir,
					'translations'            => $translations,
					'reload'                  => $uploader_settings['filr_frontend_uploader_reload']
				) );
			}
		}
	}

	/**
	 * Register a shortcode for displaying the frontend uploader.
	 *
	 * @param array $atts array of possible attributes.
	 *
	 * @return string
	 */
	public function add_shortcode( $atts ): string {

		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$uploader_settings = wp_parse_args( get_option( 'filr_frontend_uploader' ), FILR_Admin::get_defaults( 'filr_frontend_uploader' ) );

			$library = '';

			if ( isset( $atts['library'] ) ) {
				$library = $atts['library'];
			}

			$status = '';

			if ( isset( $atts['status'] ) ) {
				$status = $atts['status'];
			}

			$viewable_role = '';

			if ( isset( $atts['viewable-role'] ) ) {
				$viewable_role = $atts['viewable-role'];
			}

			$viewable_email = '';

			if ( isset( $atts['viewable-email'] ) ) {
				$viewable_email = $atts['viewable-email'];
			}

			// Check if we should restrict by user.
			if ( isset( $atts['user-email'] ) || isset( $atts['user-role'] ) ) {
				$allowed_emails = '';

				if ( isset( $atts['user-email'] ) ) {
					$allowed_emails = $atts['user-email'];
				}

				$allowed_role = '';

				if ( isset( $atts['user-role'] ) ) {
					$allowed_role = $atts['user-role'];
				}

				// check if user allowed to get file.
				$user         = wp_get_current_user();
				$user_allowed = true;

				// clean email field.
				if ( strpos( $allowed_emails, ' ' ) ) {
					str_replace( ' ', '', $allowed_emails );
				}

				// check allowed mail or mails.
				if ( ! empty( $allowed_emails ) ) {
					if ( strpos( $allowed_emails, ',' ) ) {
						$allowed_emails = explode( ',', $allowed_emails );

						if ( ! in_array( $user->user_email, $allowed_emails, true ) ) {
							$user_allowed = false;
						}
					} else {
						if ( $user->user_email !== $allowed_emails ) {
							$user_allowed = false;
						}
					}
				}

				if ( ! empty( $allowed_role ) && 'no-role' !== $allowed_role ) {
					if ( ! in_array( $allowed_role, $user->roles, true ) ) {
						$user_allowed = false;
					}
				}

				if ( current_user_can( 'administrator' ) ) {
					$user_allowed = true;
				}

				if ( ! $user_allowed ) {
					return '';
				}
			}

			// Get libraries and folders.
			$libraries = get_terms( array( 'taxonomy' => 'filr-lists', 'hide_empty' => false ) );

			$args = array(
				'numberposts' => - 1,
				'post_type'   => 'filr',
				'meta_key'    => 'is-folder',
				'meta_value'  => 'true',
				'post_status' => 'publish',
				'fields'      => 'ids',

			);

			// If we are in a specific library, only show available folders.
			if ( isset( $atts['library'] ) ) {
				$args = array(
					'numberposts' => - 1,
					'post_type'   => 'filr',
					'meta_key'    => 'is-folder',
					'meta_value'  => 'true',
					'post_status' => 'publish',
					'fields'      => 'ids',
					'tax_query'   => array(
						array(
							'taxonomy' => 'filr-lists',
							'field'    => 'slug',
							'terms'    => $atts['library'],
							'operator' => 'IN',
						)
					)
				);
			}

			$folders = get_posts( $args );

			ob_start();
			?>
            <div class="filr-frontend-uploader-container">
                <div class="filr-frontend-meta">
					<?php if ( 'on' === $uploader_settings['filr_frontend_uploader_activate_filename'] ) : ?>
                        <p>
                            <label for="file-filename"><?php esc_html_e( 'Filename', 'filr' ); ?></label>
                            <input aria-labelledby="file-filename" id="file-filename" name="file-filename" type="text">
                        </p>
					<?php endif; ?>
					<?php if ( 'on' === $uploader_settings['filr_frontend_uploader_activate_libraries'] && ! isset( $atts['library'] ) ) : ?>
                        <p>
                            <label for="file-library"><?php esc_html_e( 'Library', 'filr' ); ?></label><br>
                            <select aria-labelledby="file-library" id="file-library" name="file-library">
								<?php foreach ( $libraries as $library_item ) : ?>
                                    <option value="<?php echo esc_attr( $library_item->term_id ); ?>"><?php echo esc_html( $library_item->name ); ?></option>
								<?php endforeach; ?>
                            </select>
                        </p>
					<?php endif; ?>
					<?php if ( 'on' === $uploader_settings['filr_frontend_uploader_activate_folders'] ) : ?>
                        <p>
                            <label for="file-folder"><?php esc_html_e( 'Folder', 'filr' ); ?></label><br>
                            <select aria-labelledby="file-folder" id="file-folder" name="file-folder">
                                <option value="0"><?php esc_html_e( 'No folder', 'filr' ); ?></option>
								<?php foreach ( $folders as $folder_id ) : ?>
                                    <option value="<?php echo esc_attr( $folder_id ); ?>"><?php echo esc_html( get_the_title( $folder_id ) ); ?></option>
								<?php endforeach; ?>
                            </select>
                        </p>
					<?php endif; ?>
                </div>
                <label for="file-upload" style="padding-left: 15px;"><?php esc_html_e( 'Upload your files', 'filr' ); ?></label><br>
                <input aria-labelledby="file-upload" id="file-upload" name="file-upload" type="file"
                       data-library="<?php echo esc_html( $library ); ?>"
                       data-status="<?php echo esc_html( $status ); ?>">
                <input type="hidden" id="viewable-role" name="viewable-role" aria-label="<?php esc_html_e('Viewable Role', 'filr'); ?>"
                       value="<?php echo esc_attr( $viewable_role ); ?>">
                <input type="hidden" id="viewable-email" name="viewable-email" aria-label="<?php esc_html_e('Viewable E-Mail', 'filr'); ?>"
                       value="<?php echo esc_attr( $viewable_email ); ?>">
                <input type="hidden" id="file-id" name="file-id" aria-label="<?php esc_html_e('File ID', 'filr'); ?>">
            </div>
            <div class="filr-frontend-uploader-notice">
                <p><?php esc_html_e( 'File successfully uploaded', 'filr' ); ?></p>
            </div>
			<?php
		}

		return ob_get_clean();
	}


	public function frontend_create_post() {

		$status = 'publish';

		$file_object_data = array(
			'post_title'   => '',
			'post_content' => '',
			'post_type'    => 'filr',
			'post_status'  => $status,
		);

		$file_id = wp_insert_post( $file_object_data );

		$response = array( 'file_id' => $file_id );

		print wp_json_encode( $response );
		exit;

	}

	/**
	 * Ajax handler to upload files via FILR_Frontend_Uploader.
	 *
	 * @return void
	 */
	public function frontend_upload_file() {

		if ( ! wp_verify_nonce( $_POST['nonce'], 'filr-frontend-uploader-nonce' ) ) {
			die();
		}

		// First create a file object and save the file id.
		$file_id          = sanitize_text_field( $_POST['file_id'] );
		$status           = sanitize_text_field( $_POST['status'] );
		$filename         = sanitize_text_field( $_POST['filename'] );
		$selected_folder  = sanitize_text_field( $_POST['folder'] );
		$library          = sanitize_text_field( $_POST['library'] );
		$selected_library = sanitize_text_field( $_POST['selected_library'] );
		$viewable_role    = sanitize_text_field( $_POST['viewable-role'] );
		$viewable_email   = sanitize_text_field( $_POST['viewable-email'] );

		if ( empty( $status ) ) {
			$status = 'publish';
		}

		// Update post status.
		wp_update_post( array(
			'ID'          => $file_id,
			'post_status' => $status
		) );

		// Insert the post into the database.
		$settings                   = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$frontend_uploader_settings = wp_parse_args( get_option( 'filr_frontend_uploader' ), FILR_Admin::get_defaults( 'filr_frontend_uploader' ) );

		// build the path.
		$uploads_directory = wp_upload_dir();
		$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );

		// if directory not exists, create one.
		if ( ! file_exists( $filr_directory ) ) {
			wp_mkdir_p( $filr_directory );
		}

		if ( $file_id ) {
			// upload file.
			$file     = 'file-upload';
			$uploader = new FILR_Uploader( $file, array(
				'uploadDir' => $filr_directory,
				'title'     => 'name'
			) );
			$data     = $uploader->upload();

			// Update file name.
			if ( isset( $filename ) && ! empty( $filename ) ) {
				$current_file = array(
					'ID'         => intval( $file_id ),
					'post_title' => esc_html( $filename ),
				);
				wp_update_post( $current_file );
			} else {
				$current_file = array(
					'ID'         => intval( $file_id ),
					'post_title' => $data['files'][0]['name'],
				);
				wp_update_post( $current_file );
			}

			// Add library term.
			if ( isset( $selected_library ) && ! empty( $selected_library ) ) {
				$term = get_term_by( 'id', $selected_library, 'filr-lists' );
				wp_set_object_terms( $file_id, $term->term_id, 'filr-lists' );
			} else {
				if ( ! empty( $library ) ) {
					$term = get_term_by( 'slug', $library, 'filr-lists' );
					wp_set_object_terms( $file_id, $term->term_id, 'filr-lists' );
				}
			}

			// Assign to folder.
			if ( isset( $selected_folder ) && ! empty( $selected_folder ) ) {
				update_post_meta( $file_id, 'assigned-folder', $selected_folder );
			}

			// Role and user restriction.
			if ( isset( $viewable_role ) && ! empty( $viewable_role ) ) {
				update_post_meta( $file_id, 'restrict-user-role', $viewable_role );
			}

			if ( isset( $viewable_email ) && ! empty( $viewable_email ) ) {
				update_post_meta( $file_id, 'restrict-user', $viewable_email );
			}

			// Maybe restrict to user.
			if ( 'on' === $frontend_uploader_settings['filr_frontend_uploader_restrict_to_user'] ) {
				$user = wp_get_current_user();
				update_post_meta( $file_id, 'restrict-user', $user->user_email );
			}

			// clean file records.
			if ( apply_filters( 'filr_auto_clean', false ) ) {
				FILR_Filesystem::file_cleaner( $file_id );
			}

			$response = array( 'file-upload' => $data );
			$encryption = get_post_meta( $file_id, 'encrypt-filename', true );


			$file_url = Filr_ZIP::create_zip_file( $file_id, $encryption ); // false if it should be named as $post->name.
			update_post_meta( $file_id, 'file-download', $file_url );

			if ( ! empty( $file_url ) ) {
				$response = array( 'file-upload' => $data, 'download_link' => $file_url );
			}

			do_action( 'filr_frontend_file_uploaded', $file_id );

			print wp_json_encode( $response );
			exit;
		}

	}

	/**
	 * Ajax handler to delete files via FILR_Uploader.
	 *
	 * @return void
	 */
	public function frontend_delete_file() {
		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$file_name = sanitize_text_field( $_POST['filename'] );

			if ( ! wp_verify_nonce( $_POST['nonce'], 'filr-frontend-uploader-nonce' ) ) {
				die();
			}

			$file    = get_page_by_title( $file_name, OBJECT, 'filr' );
			$file_id = $file->ID;

			$settings = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );

			// build the path.
			$uploads_directory = wp_upload_dir();
			$filr_directory    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $settings['filr_download_directory'], $file_id );

			// delete file from file upload array in post meta.
			$existing_files = get_post_meta( $file_id, 'file-upload', true );
			$new_files      = array();

			if ( $existing_files['files'] ) {
				foreach ( $existing_files['files'] as $file ) {
					if ( $file['name'] !== $file_name ) {
						$new_files[] = $file;
					}
				}
			}

			// Update new array of files in metadata.
			$existing_files['files'] = $new_files;
			update_post_meta( $file_id, 'file-upload', $existing_files );

			$encryption = get_post_meta( $file_id, 'encrypt-filename', true );

			// if more than one file we need to regenerate zip too, otherwise update download link meta only.
			if ( $existing_files['files'] ) {
				if ( count( $existing_files['files'] ) > 1 ) {
					$file_url = Filr_ZIP::create_zip_file( $file_id, $encryption ); // false if it should be named as $post->name.
					update_post_meta( $file_id, 'file-download', $file_url );
				} else {
					$file_url = $uploads_directory['baseurl'] . '/' . $settings['filr_download_directory'] . '/' . $file_id . '/' . $existing_files['files'][0]['name'];
					update_post_meta( $file_id, 'file-download', $file_url );
					// delete zip meta if exists.
					delete_post_meta( $file_id, 'file-zip' );
				}
			} else {
				delete_post_meta( $file_id, 'file-download' );
				if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
					// delete zip meta if exists.
					delete_post_meta( $file_id, 'file-zip' );
				}
			}

			// delete the file from the filesystem.
			if ( isset( $file_name ) ) {
				$deletable_file = $filr_directory . str_replace( array( DIRECTORY_SEPARATOR, '\\' ), '', $file_name );

				if ( file_exists( $deletable_file ) ) {
					unlink( $deletable_file );
				}
			}

			// Delete file object.
			wp_delete_post( $file_id, true );

			// clean file records.
			if ( apply_filters( 'filr_auto_clean', false ) ) {
				FILR_Filesystem::file_cleaner( $file_id );
			}

			$response = array( 'delete' => true );

			if ( ! empty( $file_url ) ) {
				$response = array( 'delete' => true, 'download_link' => $file_url );
			}
			print wp_json_encode( $response );
			exit;
		}
	}


	/**
	 * Add dynamic styles if filr-uploader shortcode exists.
	 *
	 * @return void
	 */
	public function add_dynamic_styles() {
		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$uploader_settings = wp_parse_args( get_option( 'filr_frontend_uploader' ), FILR_Admin::get_defaults( 'filr_frontend_uploader' ) );
			?>
			<?php if ( shortcode_exists( 'filr' ) ) : ?>
                <style>
                    .fileuploader-icon-main::before {
                        content: "<?php echo esc_html( $uploader_settings['filr_frontend_uploader_icon'] ); ?>";
                    }

                    .fileuploader-icon-main {
                        color: <?php echo esc_html( $uploader_settings['filr_frontend_uploader_icon_color'] ); ?>;
                    }

                    .fileuploader-input .fileuploader-input-button, .fileuploader-popup .fileuploader-popup-header .fileuploader-popup-button.button-success {
                        background: <?php echo esc_html( $uploader_settings['filr_frontend_uploader_button_color'] ); ?>;
                    }
                </style>
			<?php endif; ?>
			<?php
		}
	}

	/**
	 * Send notification email after frontend submission.
	 *
	 * @param int $file_id current post id.
	 *
	 * @return void
	 */
	public function send_notification_email( int $file_id ) {
		$uploader_settings = wp_parse_args( get_option( 'filr_frontend_uploader' ), FILR_Admin::get_defaults( 'filr_frontend_uploader' ) );

		if ( 'on' === $uploader_settings['filr_frontend_uploader_send_notification'] ) {
			// Preparing the headers.
			$headers = array( 'Content-Type: text/html; charset=UTF-8' );

			// Preparing mail content.
			$link  = '< href="' . get_permalink( $file_id ) . '">' . esc_html__( 'View submitted File', 'filr' ) . '</a>';
			$title = esc_html( get_the_title( $file_id ) );

			$subject = $uploader_settings['filr_frontend_uploader_notification_subject'];
			$body    = str_replace( array( '[entry-link]', '[entry-title]' ), array(
				$link,
				$title
			), $uploader_settings['filr_frontend_uploader_notification'] );

			// Now we can send the mail.
			wp_mail( $uploader_settings['filr_frontend_uploader_notification_recipient'], $subject, $body, $headers );
		}
	}
}
