<?php

namespace filr;

/**
 * Admin Meta Class
 */
class FILR_Meta {

	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of FILR_Meta.
	 *
	 * @return object
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Constructor for FILR_Meta.
	 */
	public function __construct() {
		add_action( 'add_meta_boxes', array( $this, 'add_metaboxes' ) );
		add_action( 'save_post', array( $this, 'save_metaboxes' ) );
		add_action( 'before_delete_post', array( $this, 'clean_files' ) );
	}

	/**
	 * Adds the meta box container.
	 *
	 * @param string $post_type array of post types.
	 *
	 * @return void
	 */
	public function add_metaboxes( string $post_type ) {
		add_meta_box( 'file-download', esc_html__( 'File Download', 'filr' ), array( $this, 'render_file_download' ), 'filr', 'normal', 'high' );
		add_meta_box( 'file-upload', esc_html__( 'File Upload', 'filr' ), array( $this, 'render_file_upload' ), 'filr', 'normal', 'high' );
		add_meta_box( 'file-management', esc_html__( 'File Management', 'filr' ), array( $this, 'render_management_options' ), 'filr', 'side' );
		add_meta_box( 'file-advanced', esc_html__( 'Advanced Options', 'filr' ), array( $this, 'render_advanced_options' ), 'filr', 'side' );
		add_meta_box( 'file-user', esc_html__( 'User Options', 'filr' ), array( $this, 'render_user_options' ), 'filr', 'side' );
	}

	/**
	 * Render file download metabox.
	 *
	 * @param \WP_POST $post The post object.
	 */
	public function render_file_download( \WP_POST $post ) {
		$download_url = get_post_meta( $post->ID, 'file-download', true );
		$is_folder    = get_post_meta( $post->ID, 'is-folder', true );
		$secure_url   = FILR_Filesystem::get_secure_url( $post->ID );

		if ( $secure_url ) {
			$download_url = $secure_url;
		}

		if ( $is_folder ) {
			?>
			<div class="filr-meta">
			<p><?php esc_html_e( "Folders don't have a download link", 'filr' ); ?></p>
			</div>
			<?php
			return;
		}
		?>
		<?php if ( $download_url ) : ?>
			<span class="filr-download-link"><code><?php echo esc_url( $download_url ); ?></code></span>
		<?php else : ?>
			<span class="filr-download-link"><?php esc_html_e( 'You have not added any files yet.', 'filr' ); ?></span>
		<?php endif; ?>

		<?php
	}

	/**
	 * Render Meta Box content.
	 *
	 * @param \WP_POST $post The post object.
	 */
	public function render_file_upload( \WP_POST $post ) {
		$is_folder       = get_post_meta( $post->ID, 'is-folder', true );
		$is_external     = get_post_meta( $post->ID, 'is-external', true );
		$external_source = get_post_meta( $post->ID, 'external-source', true );

		if ( $is_folder ) {
			?>
			<div class="filr-meta">
			<p><?php esc_html_e( "Folders don't have an upload option", 'filr' ); ?></p>
			</div>
			<?php
			return;
		}

		if ( $is_external ) {
			?>
			<div class="filr-meta">
				<p>
					<label for="external-source"><?php esc_html_e( 'External Source', 'filr' ); ?></label>
					<input id="external-source" aria-labelledby="external-source" name="external-source" placeholder="<?php echo esc_url( get_bloginfo('url') ); ?>/sample-file.png" type="url" value="<?php echo esc_url( $external_source ); ?>">
				</p>
			</div>
			<?php
			return;
		}

		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
		?>
		<div class="filr-meta">
			<input id="file-upload" aria-label="<?php esc_html_e('File Upload', 'filr' ); ?>" name="file-upload" type="file">
		</div>
		<?php
		} else {
			?>
	<div class="filr-meta">
			<input id="file-upload" aria-label="<?php esc_html_e('File Upload', 'filr' ); ?>" name="file-upload" type="file" data-fileuploader-limit="1">
		</div>
		<div class="filr-meta filr-admin">
				<p><small><?php esc_html_e( 'With FILR Pro you can upload multiple files and automatically zip them.', 'filr' ); ?></small></p>
		</div>
			<?php
		}
	}

	/**
	 * Render file management metabox.
	 *
	 * @param \WP_POST $post The post object.
	 */
	public function render_management_options( \WP_POST $post ) {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			wp_nonce_field( 'filr_nonce_check', 'filr_nonce_check_value' );

			$meta_fields = array(
				'is-folder'       => get_post_meta( $post->ID, 'is-folder', true ),
				'is-external'     => get_post_meta( $post->ID, 'is-external', true ),
				'external-size'   => get_post_meta( $post->ID, 'external-size', true ),
				'assigned-folder' => get_post_meta( $post->ID, 'assigned-folder', true ),
				'version'         => get_post_meta( $post->ID, 'version', true ),
			);

			if ( $meta_fields['is-folder'] ) {
				$filename_checked = 'checked="checked"';
			} else {
				$filename_checked = '';
			}

			if ( $meta_fields['is-external'] ) {
				$external_checked = 'checked="checked"';
			} else {
				$external_checked = '';
			}

			// Files marked as folders.
			$args = array(
				'post_type'  => 'filr',
				'posts_per_page' => -1,
				'meta_key'   => 'is-folder',
				'meta_value' => 'true',
			);

			$folders = get_posts( $args );

			?>
			<div class="filr-meta filr-admin">
				<p>
					<span><?php esc_html_e( 'Use as folder instead of a file.', 'filr' ); ?></span>
					<label class="switch" for="is-folder">
							<input type="checkbox" aria-label="<?php esc_html_e('Use as folder', 'filr' ); ?>" class="toggle-checkbox" id="is-folder" name="is-folder" value="true" <?php echo $filename_checked; ?> />
						<span class="slider round"></span>
					</label>
				</p>
				<p>
					<span><?php esc_html_e( 'Use external source instead.', 'filr' ); ?></span>
					<label class="switch" for="is-external">
							<input type="checkbox" aria-label="<?php esc_html_e('Use as external source', 'filr' ); ?>" class="toggle-checkbox" id="is-external" name="is-external" value="true" <?php echo $external_checked; ?> />
						<span class="slider round"></span>
					</label>
				</p>
				<p>
					<label for="version"><?php esc_html_e( 'Version Number', 'filr' ); ?></label>
					<input id="version" aria-labelledby="version" name="version" type="text" value="<?php echo esc_html( $meta_fields['version'] ); ?>">
				</p>
				<?php if ( $meta_fields['is-external'] ) : ?>
					<p>
						<label for="external-size"><?php esc_html_e( 'External file size', 'filr' ); ?></label>
						<input id="external-size" aria-labelledby="external-size" name="external-size" type="text" value="<?php echo esc_html( $meta_fields['external-size'] ); ?>">
					</p>
				<?php endif; ?>
				<p>
					<label for="assigned-folder"><?php esc_html_e( 'Assign to folder', 'filr' ); ?></label>
					<select aria-labelledby="assigned-folder" name="assigned-folder" id="assigned-folder">
						<?php if ( empty( $meta_fields['assigned-folder'] ) ) : ?>
							<option selected="selected" value="0"><?php echo esc_html_e( 'No folder assigned', 'filr' ); ?></option>
						<?php else : ?>
							<option value="0"><?php echo esc_html_e( 'No folder assigned', 'filr' ); ?></option>
						<?php endif; ?>
						<?php foreach ( $folders as $folder ) : ?>
							<?php if ( $meta_fields['assigned-folder'] == $folder->ID ) : ?>
								<option selected="selected" value="<?php echo esc_attr( $folder->ID ); ?>"><?php echo esc_html( $folder->post_title ); ?></option>
							<?php else : ?>
								<option value="<?php echo esc_attr( $folder->ID ); ?>"><?php echo esc_html( $folder->post_title ); ?></option>
							<?php endif; ?>
						<?php endforeach; ?>
					</select>
				</p>
			</div>
			<?php
		} else {
			?>
			<div class="filr-meta filr-admin">
				<p><small><?php esc_html_e( 'With FILR Pro you can create folders and attach files to it.', 'filr' ); ?></small></p>
			</div>
			<?php
		}
	}

	/**
	 * Render file options metabox.
	 *
	 * @param \WP_POST $post The post object.
	 */
	public function render_advanced_options( \WP_POST $post ) {
		$is_folder = get_post_meta( $post->ID, 'is-folder', true );

		if ( $is_folder ) {
			?>
			<div class="filr-meta">
			<p><?php esc_html_e( "Folders don't have advanced options", 'filr' ); ?></p>
			</div>
			<?php
			return;
		}
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			wp_nonce_field( 'filr_nonce_check', 'filr_nonce_check_value' );

			$meta_fields = array(
				'expire-date'      => get_post_meta( $post->ID, 'expire-date', true ),
				'expire-download'  => get_post_meta( $post->ID, 'expire-download', true ),
				'zip-password'     => get_post_meta( $post->ID, 'zip-password', true ),
				'encrypt-filename' => get_post_meta( $post->ID, 'encrypt-filename', true ),
			);

			if ( $meta_fields['encrypt-filename'] ) {
				$filename_checked = 'checked="checked"';
			} else {
				$filename_checked = '';
			}
			?>
			<div class="filr-meta filr-admin">
				<p>
					<label for="expire-date"><?php esc_html_e( ' Expire by date', 'filr' ); ?></label>
					<input id="expire-date" aria-labelledby="expire-date" name="expire-date" type="date" value="<?php echo esc_html( $meta_fields['expire-date'] ); ?>">
				</p>
				<p>
					<label for="expire-download"><?php esc_html_e( ' Expire by downloads', 'filr' ); ?></label>
					<input id="expire-download" aria-labelledby="expire-download" name="expire-download" type="number" value="<?php echo esc_html( $meta_fields['expire-download'] ); ?>">
				</p>
				<p>
					<label for="zip-password"><?php esc_html_e( ' Password for ZIP', 'filr' ); ?></label>
					<input id="zip-password" aria-labelledby="zip-password" name="zip-password" type="text" value="<?php echo esc_html( $meta_fields['zip-password'] ); ?>">
				</p>
				<p>
                    <span><?php esc_html_e( 'Encrypt Filename', 'filr' ); ?></span>
					<label class="switch" for="encrypt-filename">
							<input aria-label="<?php esc_html_e('Encrypt Filename', 'filr' ); ?>" type="checkbox" class="toggle-checkbox" id="encrypt-filename" name="encrypt-filename" value="true" <?php echo $filename_checked; ?> />
						<span class="slider round"></span>
					</label>
				</p>
			</div>
			<?php
		} else {
			?>
			<div class="filr-meta filr-admin">
				<p><small><?php echo esc_html_e( 'With FILR Pro you can activate encryption, expire your files by date or number of downloads.', 'filr' ); ?></small></p>
			</div>
			<?php
		}
	}

	/**
	 * Render user options metabox.
	 *
	 * @param \WP_POST $post The post object.
	 */
	public function render_user_options( \WP_POST $post ) {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			wp_nonce_field( 'filr_nonce_check', 'filr_nonce_check_value' );

			$meta_fields = array(
				'restrict-user'      => get_post_meta( $post->ID, 'restrict-user', true ),
				'restrict-user-role' => get_post_meta( $post->ID, 'restrict-user-role', true ),
			);

			// get available user roles.
			$roles = array(
				array(
					'role' => 'no-role',
					'name' => __( 'No restriction', 'filr' ),
				),
			);

			$editable_roles = get_editable_roles();

			foreach ( $editable_roles as $role => $details ) {
				$data['role'] = esc_attr( $role );
				$data['name'] = translate_user_role( $details['name'] );

				// Exclude administrator from selection.
				if ( 'administrator' !== $data['role'] ) {
					$roles[] = $data;
				}
			}
			?>
			<div class="filr-meta filr-admin">
				<p>
					<label for="restrict-user"><?php esc_html_e( ' Restrict to user by email', 'filr' ); ?></label>
					<textarea aria-labelledby="restrict-user" id="restrict-user" name="restrict-user" type="email" placeholder="<?php echo esc_html( get_option( 'admin_email' ) ); ?>"><?php echo esc_html( $meta_fields['restrict-user'] ); ?></textarea>
					<br>
					<small><?php esc_html_e( 'You can  add a comma separated list with multiple E-Mail addresses here.', 'filr' ); ?></small>
				</p>
				<p>
					<label for="restrict-user-role"><?php esc_html_e( ' Restrict to user role', 'filr' ); ?></label>
					<select aria-labelledby="restrict-user-role" name="restrict-user-role" id="restrict-user-role">
						<?php foreach ( $roles as $role ) : ?>
							<?php if ( $meta_fields['restrict-user-role'] === $role['role'] ) : ?>
								<option selected="selected" value="<?php echo esc_attr( $role['role'] ); ?>"><?php echo esc_html( $role['name'] ); ?></option>
							<?php else : ?>
								<option value="<?php echo esc_attr( $role['role'] ); ?>"><?php echo esc_html( $role['name'] ); ?></option>
							<?php endif; ?>
						<?php endforeach; ?>
					</select>
					<br>
					<small><?php esc_html_e( 'Administrators have access to all files.', 'filr' ); ?></small>
				</p>
			</div>
			<?php
		} else {
			?>
			<div class="filr-meta filr-admin">
				<p><small><?php esc_html_e( 'With FILR Pro you can restrict files by user email or user role.', 'filr' ); ?></small></p>
			</div>
			<?php
		}
	}

	/**
	 * Save the meta when the post is saved.
	 *
	 * @param int $post_id The ID of the post being saved.
	 */
	public function save_metaboxes( int $post_id ): int {

		// Check if our nonce is set.
		if ( ! isset( $_POST['filr_nonce_check_value'] ) ) {
			return $post_id;
		}

		// Verify that the nonce is valid.
		if ( ! wp_verify_nonce( $_POST['filr_nonce_check_value'], 'filr_nonce_check' ) ) {
			return $post_id;
		}

		// If this is an autosave, our form has not been submitted, so we don't want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}

		// Check the user's permissions.
		if ( ! current_user_can( 'edit_post', $post_id ) ) {
			return $post_id;
		}

		// check if files in upload, otherwise delete download link.
		$upload = get_post_meta( $post_id, 'file-upload', true );

		if ( isset( $upload ) && ! empty( $upload ) ) {
			if ( count( $upload['files'] ) < 1 ) {
				delete_post_meta( $post_id, 'file-download' );
			}
		}
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			if ( isset( $_POST['expire-date'] ) && ! empty( $_POST['expire-date'] ) ) {
				update_post_meta( $post_id, 'expire-date', sanitize_text_field( $_POST['expire-date'] ) );
			} else {
				delete_post_meta( $post_id, 'expire-date' );
			}

			if ( isset( $_POST['expire-download'] ) ) {
				update_post_meta( $post_id, 'expire-download', $_POST['expire-download'] );
			} else {
				delete_post_meta( $post_id, 'expire-download' );
			}

			if ( isset( $_POST['zip-password'] ) && ! empty( $_POST['zip-password'] ) ) {
				update_post_meta( $post_id, 'zip-password', sanitize_text_field( $_POST['zip-password'] ) );
			} else {
				delete_post_meta( $post_id, 'zip-password' );
			}

			if ( isset( $_POST['encrypt-filename'] ) && ! empty( $_POST['encrypt-filename'] ) ) {
				update_post_meta( $post_id, 'encrypt-filename', $_POST['encrypt-filename'] );
				FILR_Filesystem::check_encryption_mode( $post_id, true );
			} else {
				update_post_meta( $post_id, 'encrypt-filename', null );
				FILR_Filesystem::check_encryption_mode( $post_id, false );
			}

			if ( isset( $_POST['is-folder'] ) && ! empty( $_POST['is-folder'] ) ) {
				update_post_meta( $post_id, 'is-folder', $_POST['is-folder'] );
				// if its a folder we delete the uploaded file.
				delete_post_meta( $post_id, 'file-download' );
				delete_post_meta( $post_id, 'file-upload' );
			} else {
				update_post_meta( $post_id, 'is-folder', null );
			}

			if ( isset( $_POST['version'] ) && ! empty( $_POST['version'] ) ) {
				update_post_meta( $post_id, 'version', sanitize_text_field( $_POST['version'] ) );
			} else {
				delete_post_meta( $post_id, 'version' );
			}

			if ( isset( $_POST['assigned-folder'] ) ) {
				update_post_meta( $post_id, 'assigned-folder', sanitize_text_field( $_POST['assigned-folder'] ) );
			}

			if ( isset( $_POST['restrict-user'] ) && ! empty( $_POST['restrict-user'] ) ) {
				update_post_meta( $post_id, 'restrict-user', sanitize_text_field( $_POST['restrict-user'] ) );
			} else {
				delete_post_meta( $post_id, 'restrict-user' );
			}

			if ( isset( $_POST['restrict-user-role'] ) && ! empty( $_POST['restrict-user-role'] ) ) {
				update_post_meta( $post_id, 'restrict-user-role', sanitize_text_field( $_POST['restrict-user-role'] ) );
			}

			if ( isset( $_POST['is-external'] ) && ! empty( $_POST['is-external'] ) ) {
				update_post_meta( $post_id, 'is-external', $_POST['is-external'] );
				// if its an external source we delete the uploaded file.
				delete_post_meta( $post_id, 'file-upload' );
			} else {
				update_post_meta( $post_id, 'is-external', null );
			}

			if ( isset( $_POST['external-source'] ) && ! empty( $_POST['external-source'] ) ) {
				update_post_meta( $post_id, 'external-source', sanitize_text_field( $_POST['external-source'] ) );
				update_post_meta( $post_id, 'file-download', $_POST['external-source'] );
			} else {
				delete_post_meta( $post_id, 'external-source' );
			}

			if ( isset( $_POST['external-size'] ) && ! empty( $_POST['external-size'] ) ) {
				update_post_meta( $post_id, 'external-size', sanitize_text_field( $_POST['external-size'] ) );
			} else {
				delete_post_meta( $post_id, 'external-size' );
			}

			// now checking the current file access values.
			$args = array(
				'post_type'      => 'filr',
				'posts_per_page' => -1,
				'fields'         => 'ids',
			);

			$files = get_posts( $args );

			foreach ( $files as $file_id ) {
				$expire_download = intval( get_post_meta( $file_id, 'expire-download', true ) );
				$expire_date     = get_post_meta( $file_id, 'expire-date', true );

				// check if file is blocked.
				FILR_Filesystem::block_file_access( $file_id, $expire_date, $expire_download );
			}
		}
        return $post_id;
	}
	/**
	 * Clean filesystem when post deleted.
	 *
	 * @param int $file_id current post id.
	 *
	 * @return void
	 */
	public function clean_files( int $file_id ) {
		global $post_type;
		if ( 'filr' != $post_type ) {
			return;
		}

		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
		}

		if ( is_null( $wp_filesystem ) ) {
			WP_Filesystem();
		}

		$uploads_directory = wp_upload_dir();
		$settings          = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
		$file_directory    = $uploads_directory['basedir']. DIRECTORY_SEPARATOR . $settings['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id;

		if ( file_exists( $file_directory ) ) {
			$wp_filesystem->delete( $file_directory, true );
		}
	}
}
