<?php

namespace filr;

/**
 * Shortcode Class
 */
class FILR_Shortcode {

	/**
	 * Contains instance or null
	 *
	 * @var object|null
	 */
	private static $instance = null;

	/**
	 * Returns instance of FILR_Shortcode.
	 *
	 * @return object
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}


	/**
	 * Constructor for FILR_Shortcode.
	 */
	public function __construct() {
		add_shortcode( 'filr', array( $this, 'add_shortcode' ) );
		add_action( 'wp_footer', array( $this, 'add_dynamic_styles' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'add_public_scripts' ) );

		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			add_action( 'wp_ajax_decrease_downloads', array( $this, 'decrease_downloads' ) );
			add_action( 'wp_ajax_nopriv_decrease_downloads', array( $this, 'decrease_downloads' ) );
		}
	}

	/**
	 * Decrease remaining downloads in post meta.
	 */
	public function decrease_downloads() {
		if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$download_id = sanitize_text_field( $_POST['post_id'] );
			$nonce       = $_POST['nonce'];

			if ( ! wp_verify_nonce( $nonce, 'filr-decrease-nonce' ) ) {
				die();
			}

			// get and update remaining downloads.
			$expire_download = get_post_meta( $download_id, 'expire-download', true );

			if ( '' !== $expire_download ) {
				if ( intval( $expire_download ) >= 0 ) {
					update_post_meta( $download_id, 'expire-download', $expire_download - 1 );
					$response = array( 'remaining' => $expire_download - 1 );
				}
			} else {
				$response = array( 'remaining' => '' );
			}

			print wp_json_encode( $response );
			exit;
		}
	}

	/**
	 * Add public scripts.
	 *
	 * @return void
	 */
	public function add_public_scripts() {
		if ( shortcode_exists( 'filr' ) ) {
			$suffix   = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
			$settings = wp_parse_args( get_option( 'filr_shortcode' ), FILR_Admin::get_defaults( 'filr_shortcode' ) );

			wp_enqueue_style( 'filr-datatable-css', FILR_URL . '/assets/datatables.min.css', false, 'all' );
			wp_enqueue_script( 'filr-datatables-js', FILR_URL . '/assets/datatables.min.js', array( 'jquery' ), false, true );
			wp_enqueue_script( 'filr-datatables-responsive-js', FILR_URL . '/assets/datatables.responsive.min.js', array( 'jquery' ), false, true );

			if ( 'on' === $settings['filr_activate_preview'] ) {
				wp_enqueue_script( 'filr-imagepreview', FILR_URL . '/assets/imagepreview.min.js', array( 'jquery' ), false, true );
			}

			wp_enqueue_script( 'filr-public', FILR_URL . '/assets/filr-public' . $suffix . '.js', array( 'jquery' ), FILR_VERSION, true );

			$empty_list = __( 'There are currently no files in this library.', 'filr' );

			if ( isset( $settings['filr_empty_list'] ) && ! empty( $settings['filr_empty_list'] ) ) {
				$empty_list = $settings['filr_empty_list'];
			}

			// Seach and pagination.
			$search = true;

			if ( 'on' === $settings['filr_deactivate_search'] ) {
				$search = false;
			}

			$pagination = true;

			if ( 'on' === $settings['filr_deactivate_pagination'] ) {
				$pagination = false;
			}

			if ( empty( $settings['filr_default_number_rows'] ) ) {
				$settings['filr_default_number_rows'] = 10;
			}


			$datatable_translations = array(
				'no_files'            => $empty_list,
				'count_files'         => esc_html__( 'Show _START_ to _END_ from _TOTAL_ files', 'filr' ),
				'show_files'          => esc_html__( 'Showing 0 to 0 of 0 files', 'filr' ),
				'filtered_files'      => esc_html__( '(filtered from _MAX_ total files)', 'filr' ),
				'available_files'     => esc_html__( 'Show _MENU_ files', 'filr' ),
				'loading_files'       => esc_html__( 'Loading..', 'filr' ),
				'search_files'        => esc_html__( 'Search..', 'filr' ),
				'no_files_found'      => esc_html__( 'No files found.', 'filr' ),
				'first_page_files'    => esc_html__( 'First page', 'filr' ),
				'last_page_files'     => esc_html__( 'Last page', 'filr' ),
				'next_page_files'     => esc_html__( 'Next page', 'filr' ),
				'previous_page_files' => esc_html__( 'Previous page', 'filr' ),
				'open_folder_text'    => esc_html__( 'Open Folder', 'filr' ),
				'close_folder_text'   => esc_html__( 'Close Folder', 'filr' ),
			);

			wp_localize_script(
				'filr-public',
				'filr_shortcode',
				array(
					'ajax_url'            => admin_url( 'admin-ajax.php' ),
					'nonce'               => wp_create_nonce( 'filr-decrease-nonce' ),
					'translations'        => $datatable_translations,
					'search'              => $search,
					'pagination'          => $pagination,
					'use_preview'         => $settings['filr_activate_preview'],
					'default_sort'        => $settings['filr_sort_rows_default'],
					'default_sort_type'   => $settings['filr_sort_rows_default_type'],
					'default_number_rows' => $settings['filr_default_number_rows'],
				)
			);
		}
	}

	/**
	 * Register a shortcode for filr table display.
	 *
	 * @param array $atts array of possible attributes.
	 *
	 * @return string
	 */
	public function add_shortcode( array $atts ): string {
		$settings = wp_parse_args( get_option( 'filr_shortcode' ), FILR_Admin::get_defaults( 'filr_shortcode' ) );

		if ( isset( $atts['list'] ) ) {
			$atts['library'] = $atts['list'];
		}

		if ( ! isset( $atts['library'] ) ) {
			return '';
		}

		$args = array(
			'post_type'      => 'filr',
			'posts_per_page' => - 1,
			'tax_query'      => array(
				array(
					'taxonomy' => 'filr-lists',
					'field'    => 'slug',
					'terms'    => $atts['library'],
					'operator' => 'IN',
				),
			),
			'post_status'    => 'publish',
		);

		// Show only files from uploader.
		$filter = '';

		if ( isset( $atts['filter'] ) && 'uploader' === $atts['filter'] ) {
			$filter = $atts['filter'];
			$user   = wp_get_current_user();

			$args = array(
				'post_type'      => 'filr',
				'posts_per_page' => - 1,
				'tax_query'      => array(
					array(
						'taxonomy' => 'filr-lists',
						'field'    => 'slug',
						'terms'    => $atts['library'],
						'operator' => 'IN',
					),
				),
				'meta_query'     => array(
					array(
						'key'   => 'restrict-user',
						'value' => $user->user_email,
					),
				),
				'post_status'    => 'publish',
			);
		}

		$args  = apply_filters( 'filr_library_args', $args, $filter );
		$files = get_posts( $args );

		// publish date title.
		$use_publish_date = false;

		if ( isset( $settings['filr_show_publish_instead_of_last_modified_date'] ) && ! empty( $settings['filr_show_publish_instead_of_last_modified_date'] ) && 'on' === $settings['filr_show_publish_instead_of_last_modified_date'] ) {
			add_filter(
				'filr_date_title',
				function ( $title ) {
					return __( 'Published', 'filr' );
				}
			);

			$use_publish_date = true;
		}

		// Buold the dynamic header and rows.
		$header_data = apply_filters( 'filr_header_columns', array(
			'file'      => array(
				'title' => esc_html__( 'File', 'filr' ),
				'hide'  => 'off',
			),
			'version'   => array(
				'title' => esc_html__( 'Version', 'filr' ),
				'hide'  => 'off',
			),
			'user'      => array(
				'title' => esc_html__( 'Uploaded by', 'filr' ),
				'hide'  => 'off',
			),
			'size'      => array(
				'title' => esc_html__( 'Size', 'filr' ),
				'hide'  => $settings['filr_hide_size_row'],
			),
			'type'      => array(
				'title' => esc_html__( 'Type', 'filr' ),
				'hide'  => $settings['filr_hide_type_row'],
			),
			'remaining' => array(
				'title' => esc_html__( 'Remaining', 'filr' ),
				'hide'  => $settings['filr_hide_remaining_row'],
			),
			'expire'    => array(
				'title' => esc_html__( 'Expires', 'filr' ),
				'hide'  => $settings['filr_hide_expires_row'],
			),
			'date'      => array(
				'title' => apply_filters( 'filr_date_title', esc_html__( 'Last Modified', 'filr' ) ),
				'hide'  => $settings['filr_hide_date_row'],
			),
			'download'  => array(
				'title' => esc_html__( 'Download', 'filr' ),
				'hide'  => 'off',
			),
		) );

		$headers = explode( '|', $settings['filr_shortcode_sort_rows'] );

		ob_start();
		?>
        <div class="filr-container">
            <table class="filr <?php echo esc_html( $atts['library'] ); ?>"
                   id="filr-library-<?php echo esc_html( $atts['library'] ); ?>">
                <thead>
                <tr>
					<?php foreach ( $headers as $th ) : ?>
						<?php if ( 'off' === $header_data[ $th ]['hide'] ) : ?>
                            <th class="<?php echo esc_attr( $th ); ?>"><?php echo esc_html( $header_data[ $th ]['title'] ); ?></th>
						<?php endif; ?>
					<?php endforeach; ?>
                </tr>
                </thead>
                <tbody>
				<?php foreach ( $files as $file ) : ?>
					<?php
					$file_id = $file->ID;
					$link    = apply_filters( 'filr_download_link', get_post_meta( $file_id, 'file-download', true ) );
					$upload  = get_post_meta( $file_id, 'file-upload', true );

					// Get the title.
					$title = esc_html( $file->post_title );

					// Get file date.
					$file_date = esc_html( $file->post_date );

					// Download button.
					$button = esc_html( apply_filters( 'filr_replace_button_name', $file->post_title ) );

					if ( isset( $settings['filr_rename_download_button'] ) && ! empty( $settings['filr_rename_download_button'] ) ) {
						$button = apply_filters( 'filr_replace_button_name', $settings['filr_rename_download_button'] );
					}

					if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {

						// Check author of post.
						$user_name = esc_html( apply_filters( 'filr_file_author', get_the_author_meta( 'display_name', $file->post_author ), $file->post_author ) );

						// Check for version numbers.
						$version = get_post_meta( $file_id, 'version', true );

						if ( empty( $version ) ) {
							$version = '';
						}

						// Get parent file id.
						$parent_file_id = $file_id;

						// Check file access.
						$expire_date     = get_post_meta( $file_id, 'expire-date', true );
						$expire_download = get_post_meta( $file_id, 'expire-download', true );

						// check if file blocked.
						$file_blocked = FILR_Filesystem::block_file_access( $file_id, $expire_date, $expire_download );

						if ( $file_blocked ) {
							continue;
						}

						// Check date expiration.
						if ( empty( $expire_date ) ) {
							$expire_date = __( 'never', 'filr' );
						} else {
							$expire_date = date_create( $expire_date );
							$expire_date = date_format( $expire_date, $settings['filr_date_format'] );
						}

						// if no download limit is set, set to INF.
						if ( '' === $expire_download ) {
							$expire_download = '∞';
						}

						// Check download expiration.
						if ( - 1 > $expire_download ) {
							continue;
						}

						// check if user allowed to get file.
						$allowed_emails = get_post_meta( $file_id, 'restrict-user', true );
						$allowed_role   = get_post_meta( $file_id, 'restrict-user-role', true );
						$user           = wp_get_current_user();
						$user_allowed   = true;

						// clean email field.
						if ( strpos( $allowed_emails, ' ' ) ) {
							str_replace( ' ', '', $allowed_emails );
						}

						// check allowed mail or mails.
						if ( isset( $allowed_emails ) && ! empty( $allowed_emails ) ) {
							if ( strpos( $allowed_emails, ',' ) ) {
								$allowed_emails = explode( ',', $allowed_emails );

								if ( ! in_array( $user->user_email, $allowed_emails, true ) ) {
									$user_allowed = false;
								}
							} else {
								if ( $user->user_email !== $allowed_emails ) {
									$user_allowed = false;
								}
							}
						}

						if ( isset( $allowed_role ) && ! empty( $allowed_role ) && 'no-role' !== $allowed_role ) {
							if ( ! in_array( $allowed_role, $user->roles, true ) ) {
								$user_allowed = false;
							}
						}

						if ( current_user_can( 'administrator' ) ) {
							$user_allowed = true;
						}

						if ( ! $user_allowed ) {
							continue;
						}

						// Hook to implement custom validation.
						$access_allowed = apply_filters( 'filr_access_allowed', true, $file_id, $user );

						if ( ! $access_allowed ) {
							continue;
						}

						if ( 'on' === $settings['filr_show_expired_downloads'] ) {
							add_filter( 'filr_show_blocked', '__return_false' );
						}


						// if assigned to folder remove it.
						$assigned_to_folder = get_post_meta( $file_id, 'assigned-folder', true );

						if ( $assigned_to_folder ) {
							continue;
						}

						// Is a folder?
						$is_folder    = get_post_meta( $file_id, 'is-folder', true );
						$folder_class = '';

						if ( $is_folder ) {
							$folder_class = 'filr-folder';
						}

						// Is a external source?
						$is_external     = get_post_meta( $file_id, 'is-external', true );
						$external_source = '';

						if ( $is_external ) {
							$external_source = get_post_meta( $file_id, 'external-source', true );
						}

						// Check if secure links are used.
						$secure_url = FILR_Filesystem::get_secure_url( $file_id );

						if ( $secure_url && ! $is_folder ) {
							$link = $secure_url;
						}
					}

					// if it's a file.
					if ( isset( $upload['files'] ) && ! empty( $upload['files'] ) ) {
						if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
							if ( count( $upload['files'] ) > 1 ) {
								// if date parameter is set.
								$date      = apply_filters( 'filr_date_output', date_format( date_create( $upload['files'][0]['date'] ), $settings['filr_date_format'] ) );
								$timestamp = date_format( date_create( $upload['files'][0]['date'] ), 'Ymd' );
								$size      = 0;
								$extension = 'ZIP';

								// calculate size of zip file.
								foreach ( $upload['files'] as $uploaded_file ) {
									$size = $size + $uploaded_file['size'];
								}

								// If preview is activated append to title.
								if ( 'on' === $settings['filr_activate_preview'] ) {
									$title = $title . '<a role="button" aria-label="' . esc_html__( 'Preview file', 'filr' ) . '" class="filr-preview" href="' . $link . '"></a>';
								}

								$file = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';

								// Publish date instead of last modified date.
								if ( $use_publish_date ) {
									// Get publish date.
									$publish_date = date_format( date_create( $file_date ), $settings['filr_date_format'] );
									$date         = $publish_date;
									$timestamp    = date_format( date_create( $file_date ), 'Ymd' );
								}
							} else {
								// If preview is activated append to title.
								if ( 'on' === $settings['filr_activate_preview'] ) {
									$title = $title . '<a role="button" aria-label="' . esc_html__( 'Preview file', 'filr' ) . '" class="filr-preview" href="' . $link . '"></a>';
								}

								$file = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button preview" data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';

								// if date parameter is set.
								$date      = apply_filters( 'filr_date_output', date_format( date_create( $upload['files'][0]['date'] ), $settings['filr_date_format'] ) );
								$timestamp = date_format( date_create( $upload['files'][0]['date'] ), 'Ymd' );
								$size      = $upload['files'][0]['size'];
								$extension = $upload['files'][0]['extension'];

								// Publish date instead of last modified date.
								if ( $use_publish_date ) {
									// Get publish date.
									$publish_date = date_format( date_create( $file_date ), $settings['filr_date_format'] );
									$date         = $publish_date;
									$timestamp    = date_format( date_create( $file_date ), 'Ymd' );
								}
							}
						} else {
							$file = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" href="' . $link . '" download>' . $button . '</a>';
							// if date parameter is set.
							$date      = apply_filters( 'filr_date_output', date_format( date_create( $upload['files'][0]['date'] ), $settings['filr_date_format'] ) );
							$size      = $upload['files'][0]['size'];
							$extension = $upload['files'][0]['extension'];

							// Publish date instead of last modified date.
							if ( $use_publish_date ) {
								// Get publish date.
								if ( ! empty( $file_date ) ) {
									$publish_date = date_format( date_create( $file_date ), $settings['filr_date_format'] );
									$date         = $publish_date;
									$timestamp    = date_format( date_create( $file_date ), 'Ymd' );
								}
							}
						}
					}

					if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
						// if it's a folder.
						if ( $is_folder ) {
							$file      = '<a role="button" aria-label="' . esc_html__( 'Open Folder', 'filr' ) . '" class="filr-folder-button" id="folder-' . $parent_file_id . '">' . esc_html__( 'Open Folder', 'filr' ) . '</a>';
							$extension = __( 'Folder', 'filr' );
							$size      = '';

							$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
							$timestamp = date_format( date_create( $file_date ), 'Ymd' );
						}

						// if it's a external source.
						if ( $is_external ) {
							// Check for target blank.
							$target = '';

							if ( 'on' === $settings['filr_target_blank_external'] ) {
								$target = 'target="_blank" ';
							}

							$file      = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '"' . $target . 'href="' . $external_source . '" download>' . $button . '</a>';
							$extension = __( 'External File', 'filr' );
							$size      = get_post_meta( $parent_file_id, 'external-size', true );

							$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
							$timestamp = date_format( date_create( $file_date ), 'Ymd' );
						}

						// Frontend Upload.
						if ( $link && empty( $upload['files'] ) && ! $is_external ) {
							// Check for target blank.
							$target = '';

							if ( 'on' === $settings['filr_target_blank_external'] ) {
								$target = 'target="_blank" ';
							}

							$file      = '<a role="button" aria-label="' . esc_html__( 'Download File', 'filr' ) . '" class="filr-button" data-expire="' . $expire_download . '" data-download="' . $file_id . '" href="' . $link . '" download>' . $button . '</a>';
							$extension = __( 'ZIP', 'filr' );

							// Calculate file size.
							$uploads_directory = wp_upload_dir();
							$status            = wp_parse_args( get_option( 'filr_status' ), FILR_Admin::get_defaults( 'filr_status' ) );
							$link_file_path    = apply_filters( 'filr_file_directory', $uploads_directory['basedir'] . DIRECTORY_SEPARATOR . $status['filr_download_directory'] . DIRECTORY_SEPARATOR . $file_id . DIRECTORY_SEPARATOR, $status['filr_download_directory'], $file_id ) . $file_id . '.zip';


							if ( file_exists( $link_file_path ) ) {
								$link_size = FILR_Filesystem::format_byte_sizes( filesize( $link_file_path ) );
							} else {
								$link_size = 0;
							}

							$date      = date_format( date_create( $file_date ), $settings['filr_date_format'] );
							$timestamp = date_format( date_create( $file_date ), 'Ymd' );
						}
					}

					// If nothing matches skip file.
					if ( empty( $upload['files'] ) && ! $is_folder && ! $is_external && ! $link ) {
						continue;
					}

					?>
					<?php if ( \filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) : ?>
                        <tr class="<?php echo esc_attr( $folder_class ); ?>">
							<?php foreach ( $headers as $td ) : ?>
								<?php if ( 'off' === $header_data[ $td ]['hide'] ) : ?>
									<?php if ( 'file' === $td ) : ?>
                                        <td class="title"><?php echo wp_kses_post( $title ); ?></td>
									<?php endif; ?>
									<?php if ( 'version' === $td ) : ?>
                                        <td class="version"><?php echo esc_html( $version ); ?></td>
									<?php endif; ?>
									<?php if ( 'user' === $td ) : ?>
                                        <td class="user"><?php echo esc_html( $user_name ); ?></td>
									<?php endif; ?>
									<?php if ( 'size' === $td ) : ?>
                                        <td class="size">
											<?php if ( $is_folder )  : ?>
                                                <span>-</span>
											<?php elseif ( $is_external )  : ?>
												<?php echo esc_html( $size ); ?>
											<?php elseif ( $link && empty( $upload['files'] ) )  : ?>
												<?php echo esc_html( $link_size ); ?>
											<?php else : ?>
												<?php echo esc_html( FILR_Filesystem::format_byte_sizes( $size ) ); ?>
											<?php endif; ?>
                                        </td>
									<?php endif; ?>
									<?php if ( 'type' === $td ) : ?>
                                        <td class="type"><?php echo esc_html( strtoupper( $extension ) ); ?></td>
									<?php endif; ?>
									<?php if ( 'remaining' === $td ) : ?>
                                        <td class="remaining">
											<?php if ( __( 'Folder', 'filr' ) !== $extension ) : ?>
												<?php echo esc_html( $expire_download ); ?>
											<?php else : ?>
                                                <span>-</span>
											<?php endif; ?>
                                        </td>
									<?php endif; ?>
									<?php if ( 'expire' === $td ) : ?>
                                        <td class="expire">
											<?php if ( __( 'Folder', 'filr' ) !== $extension ) : ?>
												<?php echo esc_html( $expire_date ); ?>
											<?php else : ?>
                                                <span>-</span>
											<?php endif; ?>
                                        </td>
									<?php endif; ?>
									<?php if ( 'date' === $td ) : ?>
                                        <td class="date" data-order="<?php echo esc_html( $timestamp ); ?>">
											<?php echo esc_html( $date ); ?>
                                        </td>
									<?php endif; ?>
									<?php if ( 'download' === $td ) : ?>
                                        <td class="download">
											<?php echo wp_kses_post( $file ); ?>
                                        </td>
									<?php endif; ?>
									<?php do_action( 'filr_custom_column', $td, $file_id ); ?>
								<?php endif; ?>
							<?php endforeach; ?>
                        </tr>
					<?php else : ?>
                        <tr>
							<?php foreach ( $headers as $td ) : ?>
								<?php if ( 'off' === $header_data[ $td ]['hide'] ) : ?>
									<?php if ( 'file' === $td ) : ?>
                                        <td class="title"><?php echo wp_kses_post( $title ); ?></td>
									<?php endif; ?>
									<?php if ( 'size' === $td ) : ?>
                                        <td class="size"><?php echo esc_html( FILR_Filesystem::format_byte_sizes( $size ) ); ?></td>
									<?php endif; ?>
									<?php if ( 'type' === $td ) : ?>
                                        <td class="type"><?php echo esc_html( strtoupper( $extension ) ); ?></td>
									<?php endif; ?>
									<?php if ( 'date' === $td ) : ?>
                                        <td data-order="<?php echo esc_html( $timestamp ); ?>"
                                            class="date"><?php echo esc_html( $date ); ?></td>
									<?php endif; ?>
									<?php if ( 'download' === $td ) : ?>
                                        <td class="download"><?php echo wp_kses_post( $file ); ?></td>
									<?php endif; ?>
								<?php endif; ?>
							<?php endforeach; ?>
                        </tr>
					<?php endif; ?>
				<?php endforeach; ?>
                </tbody>
            </table>
        </div>
		<?php
		if ( filr_fs()->is_plan_or_trial__premium_only( 'pro' ) ) {
			$folders = array();

			if ( isset( $files ) && ! empty( $files ) ) {
				foreach ( $files as $file ) {
					// Is a folder?
					$is_folder = get_post_meta( $file->ID, 'is-folder', true );
					if ( $is_folder ) {
						$folders[] = $file->ID;
					}
				}
			}
			?>
			<?php if ( isset( $folders ) && ! empty( $folders ) ) : ?>
				<?php sort( $folders ); ?>
				<?php foreach ( $folders as $folder_id ) : ?>
                    <div class="filr-container">
						<?php echo do_shortcode( '[filr-folder folder="' . $folder_id . '" folder-name="' . get_the_title( $folder_id ) . '"]' ); ?>
                    </div>
                    <style>
                        #filr-folder-<?php echo esc_attr( $folder_id ); ?>_wrapper {
                            visibility: hidden;
                        }
                    </style>
				<?php endforeach; ?>
			<?php endif; ?>
			<?php
		}

		return ob_get_clean();
	}

	/**
	 * Add dynamic styles if filr shortcode exists.
	 *
	 * @return void
	 */
	public function add_dynamic_styles() {

		$settings = wp_parse_args( get_option( 'filr_shortcode' ), FILR_Admin::get_defaults( 'filr_shortcode' ) );
		?>
		<?php if ( shortcode_exists( 'filr' ) ) : ?>
            <style>
                .filr-container table {
                    width: 100%;
                    border-collapse: collapse;
                    overflow: hidden;
                    border: none;
                }

                .filr-container .dataTables_wrapper {
                    background: #fafafa;
                    padding: 20px;
                    border-radius: <?php echo esc_html( $settings['filr_table_border_radius'] ); ?>px;
                }

                .filr-container .dataTables_wrapper .dataTables_filter input {
                    border: 1px solid #000;
                    border-radius: 0;
                    padding: 5px;
                    background-color: white;
                    margin-right: 0;
                }

                .filr-container .dataTables_info, .filr-container .paginate_button {
                    font-size: 0.8em;
                    color: <?php echo esc_html( $settings['filr_tr_font_color'] ); ?>;
                }

                .filr-container table, .filr-container table td, .filr-container table th, .filr-container table tr {
                    border: none;
                }

                .filr-container table thead th {
                    background: <?php echo esc_html( $settings['filr_thead_background'] ); ?>;
                    color: <?php echo esc_html( $settings['filr_thead_font_color'] ); ?>;
                    text-align: left;
                    padding: 20px;
                    border-bottom: solid 1px;
                }

                .filr-container table tbody tr {
                    background: <?php echo esc_html( $settings['filr_tr_background'] ); ?>;
                    color: <?php echo esc_html( $settings['filr_tr_font_color'] ); ?>;
                }

                .filr-container table tbody tr:nth-child(even) {
                    background-color: <?php echo esc_html( $settings['filr_tr_even_background'] ); ?>;
                }

                .filr-container table tbody tr:hover {
                    background-color: <?php echo esc_html( $settings['filr_tr_hover_background'] ); ?>;
                    transition: 0.25s;
                }

                .filr-container table tbody td {
                    position: relative;
                    padding: 20px;
                }

                .filr-container table tbody td a.filr-button {
                    background: <?php echo esc_html( $settings['filr_tr_button_background_color'] ); ?>;
                    padding: 10px 15px;
                    transition: 0.5s;
                    color: <?php echo esc_html( $settings['filr_tr_button_color'] ); ?>;
                    text-align: center;
                    text-decoration: none;
                }

                .filr-container table tbody .filr-folder-button, .filr-back {
                    background: <?php echo esc_html( $settings['filr_open_folder_button_color'] ); ?>;
                    padding: 10px 15px;
                    transition: 0.5s;
                    color: <?php echo esc_html( $settings['filr_tr_button_color'] ); ?>;
                    text-align: center;
                    text-decoration: none;
                    cursor: pointer;
                }

                .filr-container table tbody .filr-folder-button-close {
                    background: <?php echo esc_html( $settings['filr_close_folder_button_color'] ); ?>;
                }

                .filr-container table.dataTable thead .sorting:after {
                    content: "";
                    background: url("<?php echo esc_url( FILR_URL ); ?>/assets/filr-sort.svg");
                    height: 15px;
                    width: 15px;
                    display: inline-block;
                    background-repeat: no-repeat;
                    background-size: cover;
                    position: relative;
                    top: 2px;
                    left: 5px;
                    transform: rotate(180deg);
                }

                .filr-container .filr-preview:after {
                    content: "";
                    background: url("<?php echo esc_url( FILR_URL ); ?>/assets/filr-preview.svg");
                    height: 20px;
                    width: 20px;
                    display: inline-block;
                    background-repeat: no-repeat;
                    background-size: cover;
                    position: relative;
                    top: 2px;
                    left: 5px;
                }

                .filr-container table.dataTable thead .sorting_asc::after {
                    content: "";
                    background: url("<?php echo esc_url( FILR_URL ); ?>/assets/filr-sort.svg");
                    height: 15px;
                    width: 15px;
                    display: inline-block;
                    background-repeat: no-repeat;
                    background-size: cover;
                    position: relative;
                    top: 2px;
                    left: 5px;
                }

                .filr-container table.dataTable thead .sorting_desc:after {
                    content: "";
                    background: url("<?php echo esc_url( FILR_URL ); ?>/assets/filr-sort.svg");
                    height: 15px;
                    width: 15px;
                    display: inline-block;
                    background-repeat: no-repeat;
                    background-size: cover;
                    position: relative;
                    top: 2px;
                    left: 5px;
                    transform: rotate(180deg);
                }

                .filr-folder-table caption {
                    background: transparent;
                    color: <?php echo esc_html( $settings['filr_tr_button_background_color'] ); ?>;
                    font-size: <?php echo esc_html( $settings['filr_folders_font_size'] ); ?>px;
                    font-weight: bold;
                    text-align: left;
                }

                #preview {
                    position: absolute;
                    max-width: 300px;
                    border: 1px solid<?php echo esc_html( $settings['filr_thead_font_color'] ); ?>;
                    background: #fff;
                    display: none;
                }

            </style>
		<?php endif; ?>
		<?php
	}
}
